/*******************************************************
 
	GUI-independent section of recompressor 

*******************************************************/

#ifndef _RECOMPRESSOR_H
#define _RECOMPRESSOR_H

#include "gui_interface.h"
#include "imagecontrol.h"
#include "rec_cb.h"
#include <string>
#include <vector>
#include <avifile.h>
#include <videoencoder.h>
#include <videodecoder.h>
#include <audioencoder.h>
#include <pthread.h>

using namespace std;
struct AudioEncoderInfo
{
    int fmt;
    int bitrate;
};    
class FormatChanger
{
public:
    virtual void adjust(BITMAPINFOHEADER&) =0;
};
class Filter: public IFilterUI
{
    int counter;
public:
    Filter() : counter(1){}
    virtual ~Filter(){}
    virtual string name() =0;		//for list of all filters
    virtual string fullname() =0;  	//includes config options
    virtual CImage* process(CImage* im, int pos) =0;   //filters may affect only a part of stream
    virtual void addref()
    {
	counter++;
    }
    virtual void release()
    {
	counter--;
	if(!counter)delete this;
    }
    virtual string save() =0;
    virtual void load(string) =0;
    virtual int id() const {return -1;}
};

class RecompressFilter: public Filter
{
    VideoEncoderInfo info;
    char* comp_frame;
    IVideoDecoder* vd;
    IVideoEncoder* ve;
public:
    RecompressFilter(const VideoEncoderInfo& in);
    ~RecompressFilter();
    string name() {return "Recompress filter";}
    string fullname() {return name();}  	
    CImage* process(CImage* im, int pos);
    void about(){}
    void config(){}
    string save();
    void load(string);
};

class RecKernel
{
friend class Filter;
public:
    enum Mode {Remove=0, Copy=1, Recompress=2};
private:
    string strLastError;
    IRecompressCallback* _cb;
    IImageControl* _ctl;
    IAviReadFile* _rf;
    IAviWriteFile* _wf;
    RecompressFilter* _recf;
    string _fn;
    string _readfn;
    struct full_stream
    {
	IAviReadStream* stream;
	int startpos;
	int endpos;
	Mode mode;
    };
    vector<full_stream> v_str;	
    int vs, as;
//    vector<full_stream> a_str;
    vector<Filter*> filter_list;
    
    vector<AudioEncoderInfo> afmt;
    vector<VideoEncoderInfo> vfmt;
    
public:
    RecKernel() :_cb(0), _ctl(0), _rf(0), _wf(0), vs(0), as(0), rec_status(0), pause_status(0), _recf(0){}
    Filter* getFilter(int id);		//creates filter registered under id
    Filter* getFilterAt(int id);	//returns filter at pos id in current list
    Filter* operator[](int);		//synonym
    int getFilterCount() const;		//size of current list
    int addFilter(Filter* fi);		//adds filter in the end of list
    int removeFilter(int id);
    int moveUpFilter(int id);
    int moveDownFilter(int id);
    
    int openFile(const char* fn);
    const char* getSrcFile(){return _readfn.c_str();}
    int setDestFile(const char* fn);
    const char* getDestFile(){return _fn.c_str();}

    
    string aboutStream(int id) const;	//first all video streams, then audio
    
    string lastError() const;
    
    int isVideoStream(int id) const;
    int isAudioStream(int id) const;
    
    AviStream::StreamType stream_type(int id) const;
    
//    
//    int getVideoInfo(int id, VideoEncoderInfo& vi) const;
//    int getAudioInfo(int id, WAVEFORMATEX& wfmtx) const;
//    
    int isStream(int id) const;//nonzero if stream
    
    
    int setStreamMode(int id, Mode mode);
    int getStreamMode(int id, Mode& mode) const;
    int getSelection(int id, int& start, int& end) const; 
    int setSelection(int id, int start, int end); 
    int setSelectionStart(int id, int start); 
    int setSelectionEnd(int id, int end); 
    double getFrameTime(int id) const;
    void setImageControl(IImageControl* ctl);//should set size & pictures if ctl!=0 & file is opened
    
    string getStreamName(int id) const;
    int getStreamCount() const;
    int seek(int delta);//in frames
    //negative seeks are very slow
    int seek_pos(int pos);//seek to nearest keyframe, return its position
    int pos() const;
    
    int setCompress(int id, const VideoEncoderInfo& vi);
    int setAudioCompress(int id, const AudioEncoderInfo& wf);
    int getCompress(int id, VideoEncoderInfo& vi) const;
    int getAudioCompress(int id, AudioEncoderInfo& wf) const;
    
    int getVideoLength() const;
    
    int set_callback(IRecompressCallback* rec_cb);
    int start_recompress();
    int pause_recompress();
    int stop_recompress();

    void redraw();
    
    int loadConfig(const char*);
    int saveConfig(const char*);
private:
    static void* RecompressThread(void* arg);
    
    pthread_t rec_thread;
    int rec_status;
    int pause_status;
    void _destruct();
};


#endif
